#ifndef _CTRAYICON_CPP
#define _CTRAYICON_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "CTrayIcon.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
Usage Example:

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void MyTrayIconProc(CTrayIcon *lpTI, HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if(uMsg == WM_CREATE)
    {
		//Init code.
    }
	else if(uMsg == WM_COMMAND)
    {
		if(LOWORD(wParam) == ID_EXIT)
        {
            DestroyWindow(hWnd);
        }
		else if(LOWORD(wParam) == ID_CANCEL)
        {
			//Do nothing.
        }
    }
    else if(uMsg == WM_SHELLNOTIFY)
    {
		if(lParam == WM_LBUTTONDBLCLK)
		{
			if(IsWindow(hMainDialog))
			{
				SetForegroundWindow(hMainDialog);
				SetActiveWindow(hMainDialog);
			}
			else{
				DialogBox(ghAppInstance, MAKEINTRESOURCE(IDD_MAIN), NULL, MainDialog);
			}
		}
    }
    else if(uMsg == lpTI->WM_TASKBARCREATED)
    {
		//Windows explorer just started up.
		//The original iocn and tip will be re-added, do what you need to do here to update it.
    }
}
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK CTrayIconProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	CTrayIcon *lpTI = (CTrayIcon *) GetProp(hWnd, "ClassProperties");

    if(uMsg == WM_SHELLNOTIFY)
    {
		if(lParam == WM_RBUTTONUP)
        {
			POINT MyPT;
			GetCursorPos(&MyPT);
            SetForegroundWindow(hWnd);
            TrackPopupMenuEx(lpTI->hPopupMenu, TPM_LEFTALIGN|TPM_RIGHTBUTTON, MyPT.x, MyPT.y, hWnd, NULL);
            PostMessage(hWnd, WM_NULL, 0, 0);
        }
    }
    else if(uMsg == WM_DESTROY)
    {
		RemoveProp(hWnd, "ClassProperties");
		PostQuitMessage(0);
    }
    else if(uMsg == WM_CLOSE)
    {
        DestroyWindow(hWnd);
    }

	if(lpTI != NULL)
	{
		if(uMsg == lpTI->WM_TASKBARCREATED)
		{
			Shell_NotifyIcon(NIM_ADD, &lpTI->OriginalTrayNotifiy);
		}

		lpTI->pUserCallback(lpTI, hWnd, uMsg, wParam, lParam); 
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef UNICODE
bool CTrayIcon::Create(int iIcon, const WCHAR *sTitle, HMENU hInPopupMenu, UserCallback lpUserCallback)
#else
bool CTrayIcon::Create(int iIcon, const char *sTitle, HMENU hInPopupMenu, UserCallback lpUserCallback)
#endif
{
    WNDCLASS MyWC;
    MSG MyMsg;

	#ifdef UNICODE
	WCHAR sClass[255];
	wsprintf(sClass, L"CTrayIcon%s", sTitle);
	#else
	char sClass[255];
	sprintf_s(sClass, sizeof(sClass), "CTrayIcon%s", sTitle);
	#endif

	hAppInstance = (HINSTANCE) GetModuleHandle(NULL);

	this->pUserCallback = lpUserCallback;

    MyWC.style         = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	MyWC.lpfnWndProc   = CTrayIconProc;
    MyWC.cbClsExtra    = 0;
    MyWC.cbWndExtra    = 0;
    MyWC.hInstance     = hAppInstance;
    MyWC.hIcon         = LoadIcon(NULL, IDI_WINLOGO);
    MyWC.hCursor       = LoadCursor(NULL, IDC_ARROW);
    MyWC.hbrBackground = GetSysColorBrush(COLOR_BTNFACE);
    MyWC.lpszMenuName  = NULL;
    MyWC.lpszClassName = sClass;

    RegisterClass(&MyWC);

    hTrayHandle = CreateWindow(
		sClass,
		sTitle,
		WS_POPUP | WS_SYSMENU,
		0, 0, 0, 0,
		NULL,
		(HMENU) NULL,
		hAppInstance,
		NULL
	);

	SetProp(hTrayHandle, "ClassProperties", this);

	memset(&this->OriginalTrayNotifiy, 0, sizeof(this->OriginalTrayNotifiy));
	this->OriginalTrayNotifiy.cbSize			= sizeof(NOTIFYICONDATA);
    this->OriginalTrayNotifiy.hWnd				= hTrayHandle;
	this->OriginalTrayNotifiy.uID				= 0;
    this->OriginalTrayNotifiy.hIcon				= LoadIcon(hAppInstance, MAKEINTRESOURCE(iIcon) );
    this->OriginalTrayNotifiy.uFlags			= NIF_ICON | NIF_TIP | NIF_MESSAGE;
    this->OriginalTrayNotifiy.uCallbackMessage	= WM_SHELLNOTIFY;

	#ifdef UNICODE
	wcscpy_s(this->OriginalTrayNotifiy.szTip, sizeof(this->OriginalTrayNotifiy.szTip), sTitle);
	#else
	strcpy_s(this->OriginalTrayNotifiy.szTip, sizeof(this->OriginalTrayNotifiy.szTip), sTitle);
	#endif

    Shell_NotifyIcon(NIM_ADD, &this->OriginalTrayNotifiy);

	HMENU hMainMenu = CreateMenu();
	hPopupMenu = hInPopupMenu;
	#ifdef UNICODE
    InsertMenu(hMainMenu, 0, MF_POPUP, (UINT)hPopupMenu, L"invisible menu");
	#else
    InsertMenu(hMainMenu, 0, MF_POPUP, (UINT)hPopupMenu, "invisible menu");
	#endif

	this->pUserCallback(this, this->hTrayHandle, WM_INITCTRAYICON, 0, 0);

    while(GetMessage(&MyMsg, NULL, 0, 0))
    {
        if(!IsWindow(hTrayHandle) || !IsDialogMessage(hTrayHandle, &MyMsg))
        {
            TranslateMessage(&MyMsg);
            DispatchMessage(&MyMsg);
        }
		Sleep(1);
    }

	UnregisterClass(sClass, hAppInstance);
	Shell_NotifyIcon(NIM_DELETE, &this->OriginalTrayNotifiy);

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CTrayIcon::Update(int iIcon)
{
	NOTIFYICONDATA MyTrayNotifiy;
	memset(&MyTrayNotifiy, 0, sizeof(MyTrayNotifiy));

	MyTrayNotifiy.cbSize= sizeof(NOTIFYICONDATA);
    MyTrayNotifiy.hWnd = hTrayHandle;
    MyTrayNotifiy.uFlags = NIF_ICON;
	MyTrayNotifiy.hIcon = LoadIcon(hAppInstance, MAKEINTRESOURCE(iIcon));

    return (Shell_NotifyIcon(NIM_MODIFY, &MyTrayNotifiy) != 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef UNICODE
bool CTrayIcon::Update(const WCHAR *sTitle)
#else
bool CTrayIcon::Update(const char *sTitle)
#endif
{
	NOTIFYICONDATA MyTrayNotifiy;
	memset(&MyTrayNotifiy, 0, sizeof(MyTrayNotifiy));

	MyTrayNotifiy.cbSize= sizeof(NOTIFYICONDATA);
    MyTrayNotifiy.hWnd = hTrayHandle;
    MyTrayNotifiy.uFlags = NIF_TIP;

	#ifdef UNICODE
	wcscpy_s(MyTrayNotifiy.szTip, sizeof(MyTrayNotifiy.szTip), sTitle);
	#else
	strcpy_s(MyTrayNotifiy.szTip, sizeof(MyTrayNotifiy.szTip), sTitle);
	#endif

    return (Shell_NotifyIcon(NIM_MODIFY, &MyTrayNotifiy) != 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef UNICODE
bool CTrayIcon::Update(int iIcon, const WCHAR *sTitle)
#else
bool CTrayIcon::Update(int iIcon, const char *sTitle)
#endif
{
	NOTIFYICONDATA MyTrayNotifiy;
	memset(&MyTrayNotifiy, 0, sizeof(MyTrayNotifiy));

	MyTrayNotifiy.cbSize= sizeof(NOTIFYICONDATA);
    MyTrayNotifiy.hWnd = hTrayHandle;
    MyTrayNotifiy.uFlags = NIF_ICON | NIF_TIP;
	MyTrayNotifiy.hIcon = LoadIcon(hAppInstance, MAKEINTRESOURCE(iIcon));

	#ifdef UNICODE
	wcsncpy_s(MyTrayNotifiy.szTip, sizeof(MyTrayNotifiy.szTip), sTitle, 63);
	#else
	strncpy_s(MyTrayNotifiy.szTip, sizeof(MyTrayNotifiy.szTip), sTitle, 63);
	#endif

	return (Shell_NotifyIcon(NIM_MODIFY, &MyTrayNotifiy) != 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CTrayIcon::Destroy(void)
{
	NOTIFYICONDATA MyTrayNotifiy;
	memset(&MyTrayNotifiy, 0, sizeof(MyTrayNotifiy));

	MyTrayNotifiy.cbSize= sizeof(NOTIFYICONDATA);
    MyTrayNotifiy.hWnd = hTrayHandle;

	SendMessage(hTrayHandle, WM_CLOSE, 0, 0);

	this->hTrayHandle = NULL;
	this->hPopupMenu = NULL;

	return (Shell_NotifyIcon(NIM_DELETE, &MyTrayNotifiy) != 0);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool CTrayIcon::DoesIconExist(void)
{
	if(this->hTrayHandle)
	{
		return(IsWindow(this->hTrayHandle) > 0);
	}
	return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
